@@ -56,9 +56,11 @@ class SINK_EXPORT Listener : public QObject
 {
     Q_OBJECT
 public:
-    Listener(const QByteArray &resourceName, const QByteArray &resourceType, QObject *parent = 0);
+    Listener(const QByteArray &resourceName, const QByteArray &resourceType, QObject *parent = nullptr);
     ~Listener();
 
+    void checkForUpgrade();
+
 signals:
     void noClients();
 
