@@ -33,7 +33,6 @@
 #include "entitybuffer.h"
 #include "log.h"
 #include "domain/applicationdomaintype.h"
-#include "domain/applicationdomaintype_p.h"
 #include "adaptorfactoryregistry.h"
 #include "definitions.h"
 #include "bufferutils.h"
@@ -123,7 +122,7 @@ KAsync::Job<qint64> Pipeline::newEntity(void const *command, size_t size)
         flatbuffers::Verifier verifyer(reinterpret_cast<const uint8_t *>(command), size);
         if (!Commands::VerifyCreateEntityBuffer(verifyer)) {
             SinkWarningCtx(d->logCtx) << "invalid buffer, not a create entity buffer";
-            return KAsync::error<qint64>(0);
+            return KAsync::error<qint64>();
         }
     }
     auto createEntity = Commands::GetCreateEntity(command);
@@ -135,7 +134,7 @@ KAsync::Job<qint64> Pipeline::newEntity(void const *command, size_t size)
         key = QByteArray(reinterpret_cast<char const *>(createEntity->entityId()->Data()), createEntity->entityId()->size());
         if (!key.isEmpty() && d->entityStore.contains(bufferType, key)) {
             SinkErrorCtx(d->logCtx) << "An entity with this id already exists: " << key;
-            return KAsync::error<qint64>(0);
+            return KAsync::error<qint64>();
         }
     }
 
@@ -149,19 +148,19 @@ KAsync::Job<qint64> Pipeline::newEntity(void const *command, size_t size)
         flatbuffers::Verifier verifyer(reinterpret_cast<const uint8_t *>(createEntity->delta()->Data()), createEntity->delta()->size());
         if (!VerifyEntityBuffer(verifyer)) {
             SinkWarningCtx(d->logCtx) << "invalid buffer, not an entity buffer";
-            return KAsync::error<qint64>(0);
+            return KAsync::error<qint64>();
         }
     }
     auto entity = GetEntity(createEntity->delta()->Data());
     if (!entity->resource()->size() && !entity->local()->size()) {
         SinkWarningCtx(d->logCtx) << "No local and no resource buffer while trying to create entity.";
-        return KAsync::error<qint64>(0);
+        return KAsync::error<qint64>();
     }
 
     auto adaptorFactory = Sink::AdaptorFactoryRegistry::instance().getFactory(d->resourceContext.resourceType, bufferType);
     if (!adaptorFactory) {
         SinkWarningCtx(d->logCtx) << "no adaptor factory for type " << bufferType << d->resourceContext.resourceType;
-        return KAsync::error<qint64>(0);
+        return KAsync::error<qint64>();
     }
 
     auto adaptor = adaptorFactory->createAdaptor(*entity);
@@ -181,7 +180,7 @@ KAsync::Job<qint64> Pipeline::newEntity(void const *command, size_t size)
     }
 
     if (!d->entityStore.add(bufferType, newEntity, replayToSource)) {
-        return KAsync::error<qint64>(0);
+        return KAsync::error<qint64>();
     }
 
     return KAsync::value(d->entityStore.maxRevision());
@@ -207,7 +206,7 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
         flatbuffers::Verifier verifyer(reinterpret_cast<const uint8_t *>(command), size);
         if (!Commands::VerifyModifyEntityBuffer(verifyer)) {
             SinkWarningCtx(d->logCtx) << "invalid buffer, not a modify entity buffer";
-            return KAsync::error<qint64>(0);
+            return KAsync::error<qint64>();
         }
     }
     auto modifyEntity = Commands::GetModifyEntity(command);
@@ -226,20 +225,20 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
     SinkTraceCtx(d->logCtx) << "Modified Entity. Type: " << bufferType << "uid: "<< key << " replayToSource: " << replayToSource;
     if (bufferType.isEmpty() || key.isEmpty()) {
         SinkWarningCtx(d->logCtx) << "entity type or key " << bufferType << key;
-        return KAsync::error<qint64>(0);
+        return KAsync::error<qint64>();
     }
     {
         flatbuffers::Verifier verifyer(reinterpret_cast<const uint8_t *>(modifyEntity->delta()->Data()), modifyEntity->delta()->size());
         if (!VerifyEntityBuffer(verifyer)) {
             SinkWarningCtx(d->logCtx) << "invalid buffer, not an entity buffer";
-            return KAsync::error<qint64>(0);
+            return KAsync::error<qint64>();
         }
     }
 
     auto adaptorFactory = Sink::AdaptorFactoryRegistry::instance().getFactory(d->resourceContext.resourceType, bufferType);
     if (!adaptorFactory) {
         SinkWarningCtx(d->logCtx) << "no adaptor factory for type " << bufferType;
-        return KAsync::error<qint64>(0);
+        return KAsync::error<qint64>();
     }
 
     auto diffEntity = GetEntity(modifyEntity->delta()->Data());
@@ -252,10 +251,26 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
         deletions = BufferUtils::fromVector(*modifyEntity->deletions());
     }
 
-    const auto current = d->entityStore.readLatest(bufferType, diff.identifier());
+    Sink::ApplicationDomain::ApplicationDomainType current;
+    bool alreadyRemoved = false;
+
+    d->entityStore.readLatest(bufferType, diff.identifier(), [&](const QByteArray &uid, const EntityBuffer &buffer) {
+        if (buffer.operation() == Sink::Operation_Removal) {
+            alreadyRemoved = true;
+        } else {
+            auto entity = Sink::ApplicationDomain::ApplicationDomainType{d->resourceContext.instanceId(), key, baseRevision, adaptorFactory->createAdaptor(buffer.entity())};
+            current = *Sink::ApplicationDomain::ApplicationDomainType::getInMemoryRepresentation<Sink::ApplicationDomain::ApplicationDomainType>(entity, entity.availableProperties());
+        }
+    });
+
+    if (alreadyRemoved) {
+        SinkWarningCtx(d->logCtx) << "Tried to modify a removed entity: " << diff.identifier();
+        return KAsync::error<qint64>();
+    }
+
     if (current.identifier().isEmpty()) {
         SinkWarningCtx(d->logCtx) << "Failed to read current version: " << diff.identifier();
-        return KAsync::error<qint64>(0);
+        return KAsync::error<qint64>();
     }
 
     //We avoid overwriting local changes that haven't been played back yet with remote modifications
@@ -294,7 +309,7 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
                 break;
             case Preprocessor::DropModification:
                 SinkTraceCtx(d->logCtx) << "Dropping modification";
-                return KAsync::error<qint64>(0);
+                return KAsync::error<qint64>();
             case Preprocessor::NoAction:
             case Preprocessor::DeleteEntity:
             default:
@@ -335,7 +350,7 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
 
     d->revisionChanged = true;
     if (!d->entityStore.modify(bufferType, current, newEntity, replayToSource)) {
-        return KAsync::error<qint64>(0);
+        return KAsync::error<qint64>();
     }
 
     return KAsync::value(d->entityStore.maxRevision());
@@ -349,7 +364,7 @@ KAsync::Job<qint64> Pipeline::deletedEntity(void const *command, size_t size)
         flatbuffers::Verifier verifyer(reinterpret_cast<const uint8_t *>(command), size);
         if (!Commands::VerifyDeleteEntityBuffer(verifyer)) {
             SinkWarningCtx(d->logCtx) << "invalid buffer, not a delete entity buffer";
-            return KAsync::error<qint64>(0);
+            return KAsync::error<qint64>();
         }
     }
     auto deleteEntity = Commands::GetDeleteEntity(command);
@@ -367,7 +382,7 @@ KAsync::Job<qint64> Pipeline::deletedEntity(void const *command, size_t size)
 
     d->revisionChanged = true;
     if (!d->entityStore.remove(bufferType, current, replayToSource)) {
-        return KAsync::error<qint64>(0);
+        return KAsync::error<qint64>();
     }
 
     return KAsync::value(d->entityStore.maxRevision());
@@ -456,7 +471,7 @@ Storage::EntityStore &Preprocessor::entityStore() const
     return *d->entityStore;
 }
 
-void Preprocessor::createEntity(const Sink::ApplicationDomain::ApplicationDomainType &entity, const QByteArray &typeName)
+void Preprocessor::createEntity(const Sink::ApplicationDomain::ApplicationDomainType &entity, const QByteArray &typeName, bool replayToSource)
 {
     flatbuffers::FlatBufferBuilder entityFbb;
     auto adaptorFactory = Sink::AdaptorFactoryRegistry::instance().getFactory(d->resourceType, typeName);
@@ -467,13 +482,24 @@ void Preprocessor::createEntity(const Sink::ApplicationDomain::ApplicationDomain
     auto entityId = fbb.CreateString(entity.identifier().toStdString());
     auto type = fbb.CreateString(typeName.toStdString());
     auto delta = Sink::EntityBuffer::appendAsVector(fbb, entityBuffer.constData(), entityBuffer.size());
-    auto location = Sink::Commands::CreateCreateEntity(fbb, entityId, type, delta);
+    auto location = Sink::Commands::CreateCreateEntity(fbb, entityId, type, delta, replayToSource);
     Sink::Commands::FinishCreateEntityBuffer(fbb, location);
 
     const auto data = BufferUtils::extractBuffer(fbb);
     d->pipeline->newEntity(data, data.size()).exec();
 }
 
+void Preprocessor::deleteEntity(const Sink::ApplicationDomain::ApplicationDomainType &entity, const QByteArray &typeName, bool replayToSource)
+{
+    flatbuffers::FlatBufferBuilder fbb;
+    auto entityId = fbb.CreateString(entity.identifier().toStdString());
+    auto type = fbb.CreateString(typeName.toStdString());
+    auto location = Sink::Commands::CreateDeleteEntity(fbb, entity.revision(), entityId, type, replayToSource);
+    Sink::Commands::FinishDeleteEntityBuffer(fbb, location);
+    const auto data = BufferUtils::extractBuffer(fbb);
+    d->pipeline->deletedEntity(data, data.size()).exec();
+}
+
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wundefined-reinterpret-cast"
 #include "moc_pipeline.cpp"
