@@ -37,35 +37,27 @@ KAsync::Job<void> ResourceControl::shutdown(const QByteArray &identifier)
     SinkTrace() << "shutdown " << identifier;
     auto time = QSharedPointer<QTime>::create();
     time->start();
-    return ResourceAccess::connectToServer(identifier)
-        .then<void, QSharedPointer<QLocalSocket>>(
-            [identifier, time](const KAsync::Error &error, QSharedPointer<QLocalSocket> socket) {
-                if (error) {
-                    SinkTrace() << "Resource is already closed.";
-                    // Resource isn't started, nothing to shutdown
-                    return KAsync::null();
+
+    auto resourceAccess = ResourceAccessFactory::instance().getAccess(identifier, ResourceConfig::getResourceType(identifier));
+    return resourceAccess->shutdown()
+        .addToContext(resourceAccess)
+        .then<void>([resourceAccess, time](KAsync::Future<void> &future) {
+            SinkTrace() << "Shutdown command complete, waiting for shutdown." << Log::TraceTime(time->elapsed());
+            if (!resourceAccess->isReady()) {
+                future.setFinished();
+                return;
+            }
+            auto guard = new QObject;
+            QObject::connect(resourceAccess.data(), &ResourceAccess::ready, guard, [&future, guard](bool ready) {
+                if (!ready) {
+                    //Protect against callback getting called twice.
+                    delete guard;
+                    future.setFinished();
                 }
-                // We can't currently reuse the socket
-                socket->close();
-                auto resourceAccess = ResourceAccessFactory::instance().getAccess(identifier, ResourceConfig::getResourceType(identifier));
-                resourceAccess->open();
-                return resourceAccess->sendCommand(Sink::Commands::ShutdownCommand)
-                    .addToContext(resourceAccess)
-                    .then<void>([resourceAccess, time](KAsync::Future<void> &future) {
-                        SinkTrace() << "Shutdown command complete, waiting for shutdown." << Log::TraceTime(time->elapsed());
-                        if (!resourceAccess->isReady()) {
-                            future.setFinished();
-                            return;
-                        }
-                        QObject::connect(resourceAccess.data(), &ResourceAccess::ready, [&future](bool ready) {
-                                if (!ready) {
-                                    future.setFinished();
-                                }
-                            });
-                    }).then([time] {
-                        SinkTrace() << "Shutdown complete." << Log::TraceTime(time->elapsed());
-                    });
             });
+        }).then([time] {
+            SinkTrace() << "Shutdown complete." << Log::TraceTime(time->elapsed());
+        });
 }
 
 KAsync::Job<void> ResourceControl::start(const QByteArray &identifier)
@@ -98,7 +90,7 @@ KAsync::Job<void> ResourceControl::flush(Flush::FlushType type, const QByteArray
     auto notifier = QSharedPointer<Sink::Notifier>::create(resourceAccess);
     auto id = createUuid();
     return KAsync::start<void>([=](KAsync::Future<void> &future) {
-            SinkTrace() << "Waiting for flush completion notification " << id;
+            SinkLog() << "Starting flush " << id;
             notifier->registerHandler([&future, id](const Notification &notification) {
                 SinkTrace() << "Received notification: " << notification.type << notification.id;
                 if (notification.type == Notification::Error && notification.code == ApplicationDomain::ResourceCrashedError) {
