@@ -24,6 +24,7 @@
 #include "store.h"
 #include "resourceaccess.h"
 #include "resource.h"
+#include "facadefactory.h"
 
 using namespace Sink;
 
@@ -79,13 +80,13 @@ typename ApplicationDomain::SinkResource::Ptr readFromConfig<ApplicationDomain::
     return object;
 }
 
-static bool matchesFilter(const QHash<QByteArray, Query::Comparator> &filter, const ApplicationDomain::ApplicationDomainType &entity)
+static bool matchesFilter(const QHash<QByteArrayList, Query::Comparator> &filter, const ApplicationDomain::ApplicationDomainType &entity)
 {
     for (const auto &filterProperty : filter.keys()) {
-        if (filterProperty == ApplicationDomain::SinkResource::ResourceType::name) {
+        if (filterProperty[0] == ApplicationDomain::SinkResource::ResourceType::name) {
             continue;
         }
-        if (!filter.value(filterProperty).matches(entity.getProperty(filterProperty))) {
+        if (!filter.value(filterProperty).matches(entity.getProperty(filterProperty[0]))) {
             return false;
         }
     }
@@ -109,7 +110,7 @@ LocalStorageQueryRunner<DomainType>::LocalStorageQueryRunner(const Query &query,
     };
 
     QObject *guard = new QObject;
-    mResultProvider->setFetcher([this, query, guard, &configNotifier, matchesTypeAndIds](const QSharedPointer<DomainType> &) {
+    mResultProvider->setFetcher([this, query, matchesTypeAndIds]() {
         const auto entries = mConfigStore.getEntries();
         for (const auto &res : entries.keys()) {
             const auto type = entries.value(res);
@@ -126,7 +127,7 @@ LocalStorageQueryRunner<DomainType>::LocalStorageQueryRunner(const Query &query,
             mResultProvider->add(entity);
         }
         // TODO initialResultSetComplete should be implicit
-        mResultProvider->initialResultSetComplete(typename DomainType::Ptr(), true);
+        mResultProvider->initialResultSetComplete(true);
         mResultProvider->complete();
     });
     if (query.liveQuery()) {
@@ -358,27 +359,46 @@ QPair<KAsync::Job<void>, typename Sink::ResultEmitter<typename ApplicationDomain
     auto ctx = parentCtx.subContext("accounts");
     auto runner = new LocalStorageQueryRunner<ApplicationDomain::SinkAccount>(query, mIdentifier, mTypeName, sConfigNotifier, ctx);
     auto monitoredResources = QSharedPointer<QSet<QByteArray>>::create();
-    runner->setStatusUpdater([runner, monitoredResources, ctx](ApplicationDomain::SinkAccount &account) {
-        Query query;
+    auto monitorResource = [monitoredResources, runner, ctx] (const QByteArray &accountIdentifier, const ApplicationDomain::SinkResource &resource, const ResourceAccess::Ptr &resourceAccess) {
+        if (!monitoredResources->contains(resource.identifier())) {
+            auto ret = QObject::connect(resourceAccess.data(), &ResourceAccess::notification, runner->guard(), [resource, runner, resourceAccess, accountIdentifier, ctx](const Notification &notification) {
+                SinkTraceCtx(ctx) << "Received notification in facade: " << notification.type;
+                if (notification.type == Notification::Status) {
+                    runner->statusChanged(accountIdentifier);
+                }
+            });
+            Q_ASSERT(ret);
+            monitoredResources->insert(resource.identifier());
+        }
+    };
+    runner->setStatusUpdater([runner, monitoredResources, ctx, monitorResource](ApplicationDomain::SinkAccount &account) {
+        Query query{Query::LiveQuery};
         query.filter<ApplicationDomain::SinkResource::Account>(account.identifier());
         query.request<ApplicationDomain::SinkResource::Account>()
              .request<ApplicationDomain::SinkResource::Capabilities>();
         const auto resources = Store::read<ApplicationDomain::SinkResource>(query);
         SinkTraceCtx(ctx) << "Found resource belonging to the account " << account.identifier() << " : " << resources;
         auto accountIdentifier = account.identifier();
+
+        //Monitor for new resources so they can be monitored as well
+        if (!runner->mResourceEmitter.contains(accountIdentifier)) {
+            auto facade = Sink::FacadeFactory::instance().getFacade<ApplicationDomain::SinkResource>();
+            Q_ASSERT(facade);
+
+            auto emitter = facade->load(query, ctx).second;
+            emitter->onAdded([=](const ApplicationDomain::SinkResource::Ptr &resource) {
+                auto resourceAccess = Sink::ResourceAccessFactory::instance().getAccess(resource->identifier(), ResourceConfig::getResourceType(resource->identifier()));
+                monitorResource(accountIdentifier, *resource, resourceAccess);
+            });
+            emitter->fetch();
+            runner->mResourceEmitter[accountIdentifier] = emitter;
+        }
+
         QList<int> states;
+        //Gather all resources and ensure they are monitored
         for (const auto &resource : resources) {
             auto resourceAccess = ResourceAccessFactory::instance().getAccess(resource.identifier(), ResourceConfig::getResourceType(resource.identifier()));
-            if (!monitoredResources->contains(resource.identifier())) {
-                auto ret = QObject::connect(resourceAccess.data(), &ResourceAccess::notification, runner->guard(), [resource, runner, resourceAccess, accountIdentifier, ctx](const Notification &notification) {
-                    SinkTraceCtx(ctx) << "Received notification in facade: " << notification.type;
-                    if (notification.type == Notification::Status) {
-                        runner->statusChanged(accountIdentifier);
-                    }
-                });
-                Q_ASSERT(ret);
-                monitoredResources->insert(resource.identifier());
-            }
+            monitorResource(accountIdentifier, resource, resourceAccess);
             states << resourceAccess->getResourceStatus();
         }
         const auto status = [&] {
@@ -391,7 +411,10 @@ QPair<KAsync::Job<void>, typename Sink::ResultEmitter<typename ApplicationDomain
             if (states.contains(ApplicationDomain::OfflineStatus)) {
                 return ApplicationDomain::OfflineStatus;
             }
-            return ApplicationDomain::ConnectedStatus;
+            if (states.contains(ApplicationDomain::ConnectedStatus)) {
+                return ApplicationDomain::ConnectedStatus;
+            }
+            return ApplicationDomain::NoStatus;
         }();
         account.setStatusStatus(status);
     });
@@ -409,7 +432,7 @@ KAsync::Job<void> AccountFacade::remove(const Sink::ApplicationDomain::SinkAccou
     //Remove all identities
     job = job.then(Store::fetch<Identity>(Sink::Query{}.filter<Identity::Account>(account)))
         .each([] (const Identity::Ptr &identity) { return Store::remove(*identity); });
- 
+
     return job.then(LocalStorageFacade<Sink::ApplicationDomain::SinkAccount>::remove(account));
 }
 
