@@ -23,19 +23,7 @@
 
 #include <signal.h>
 #ifndef Q_OS_WIN
-#include <execinfo.h>
-#endif
-#include <csignal>
-#include <iostream>
-#include <cstdlib>
-#include <ostream>
-#include <sstream>
-#include <thread>
-#include <chrono>
-#ifndef Q_OS_WIN
 #include <unistd.h>
-#include <cxxabi.h>
-#include <dlfcn.h>
 #else
 #include <io.h>
 #include <process.h>
@@ -45,104 +33,13 @@
 #include "log.h"
 #include "test.h"
 #include "definitions.h"
+#include "backtrace.h"
 #ifdef Q_OS_OSX
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
-static Listener *listener = nullptr;
 
-//Print a demangled stacktrace
-void printStacktrace()
-{
-#ifndef Q_OS_WIN
-    int skip = 1;
-	void *callstack[128];
-	const int nMaxFrames = sizeof(callstack) / sizeof(callstack[0]);
-	char buf[1024];
-	int nFrames = backtrace(callstack, nMaxFrames);
-	char **symbols = backtrace_symbols(callstack, nFrames);
 
-	std::ostringstream trace_buf;
-	for (int i = skip; i < nFrames; i++) {
-		// printf("%s\n", symbols[i]);
-		Dl_info info;
-		if (dladdr(callstack[i], &info) && info.dli_sname) {
-			char *demangled = NULL;
-			int status = -1;
-			if (info.dli_sname[0] == '_') {
-				demangled = abi::__cxa_demangle(info.dli_sname, NULL, 0, &status);
-            }
-			snprintf(buf, sizeof(buf), "%-3d %*p %s + %zd\n",
-					 i, int(2 + sizeof(void*) * 2), callstack[i],
-					 status == 0 ? demangled :
-					 info.dli_sname == 0 ? symbols[i] : info.dli_sname,
-					 (char *)callstack[i] - (char *)info.dli_saddr);
-			free(demangled);
-		} else {
-			snprintf(buf, sizeof(buf), "%-3d %*p %s\n",
-					 i, int(2 + sizeof(void*) * 2), callstack[i], symbols[i]);
-		}
-		trace_buf << buf;
-	}
-	free(symbols);
-	if (nFrames == nMaxFrames) {
-		trace_buf << "[truncated]\n";
-    }
-    std::cerr << trace_buf.str();
-#endif
-}
-
-static int sCounter = 0;
-
-void crashHandler(int signal)
-{
-    //Guard against crashing in here
-    if (sCounter > 1) {
-        std::_Exit(EXIT_FAILURE);
-    }
-    sCounter++;
-
-    if (signal == SIGABRT) {
-        std::cerr << "SIGABRT received\n";
-    } else if (signal == SIGSEGV) {
-        std::cerr << "SIGSEV received\n";
-    } else {
-        std::cerr << "Unexpected signal " << signal << " received\n";
-    }
-
-    printStacktrace();
-
-    //Get the word out that we're going down
-    listener->emergencyAbortAllConnections();
-
-    std::fprintf(stdout, "Sleeping for 10s to attach a debugger: gdb attach %i\n", getpid());
-    std::this_thread::sleep_for(std::chrono::seconds(10));
-
-    // std::system("exec gdb -p \"$PPID\" -ex \"thread apply all bt\"");
-    // This only works if we actually have xterm and X11 available
-    // std::system("exec xterm -e gdb -p \"$PPID\"");
-
-    std::_Exit(EXIT_FAILURE);
-}
-
-void terminateHandler()
-{
-    std::exception_ptr exptr = std::current_exception();
-    if (exptr != 0)
-    {
-        // the only useful feature of std::exception_ptr is that it can be rethrown...
-        try {
-            std::rethrow_exception(exptr);
-        } catch (std::exception &ex) {
-            std::fprintf(stderr, "Terminated due to exception: %s\n", ex.what());
-        } catch (...) {
-            std::fprintf(stderr, "Terminated due to unknown exception\n");
-        }
-    } else {
-        std::fprintf(stderr, "Terminated due to unknown reason :(\n");
-    }
-    std::abort();
-}
 
 /*
  * We capture all qt debug messages in the same process and feed it into the sink debug system.
@@ -231,10 +128,7 @@ int main(int argc, char *argv[])
         raise(SIGSTOP);
 #endif
     } else {
-        // For crashes
-        std::signal(SIGSEGV, crashHandler);
-        std::signal(SIGABRT, crashHandler);
-        std::set_terminate(terminateHandler);
+        Sink::installCrashHandler();
     }
 
     qInstallMessageHandler(qtMessageHandler);
@@ -293,7 +187,8 @@ int main(int argc, char *argv[])
         return -1;
     }
 
-    listener = new Listener(instanceIdentifier, resourceType, &app);
+    auto listener = new Listener(instanceIdentifier, resourceType, &app);
+    Sink::setListener(listener);
     listener->checkForUpgrade();
 
     QObject::connect(&app, &QCoreApplication::aboutToQuit, listener, &Listener::closeAllConnections);
