@@ -171,6 +171,30 @@ void MailSyncTest::testListRemovedFolder()
     VERIFYEXEC(job);
 }
 
+void MailSyncTest::testListRemovedFullFolder()
+{
+    createFolder({"testRemoval"});
+    createMessage({"testRemoval"}, newMessage("mailToRemove"));
+
+    Sink::Query query;
+    query.resourceFilter(mResourceInstanceIdentifier);
+    query.request<Folder::Name>();
+
+    VERIFYEXEC(Store::synchronize(query));
+    VERIFYEXEC(ResourceControl::flushMessageQueue(mResourceInstanceIdentifier));
+    QCOMPARE(Sink::Store::read<Folder>(Sink::Query{}.filter<Folder::Name>("testRemoval")).size(), 1);
+    QCOMPARE(Sink::Store::read<Mail>(Sink::Query{}.filter<Mail::Subject>("mailToRemove")).size(), 1);
+
+    removeFolder({"testRemoval"});
+
+    // Ensure all local data is processed
+    VERIFYEXEC(Store::synchronize(query));
+    VERIFYEXEC(ResourceControl::flushMessageQueue(mResourceInstanceIdentifier));
+
+    QCOMPARE(Sink::Store::read<Folder>(Sink::Query{}.filter<Folder::Name>("testRemoval")).size(), 0);
+    QCOMPARE(Sink::Store::read<Mail>(Sink::Query{}.filter<Mail::Subject>("mailToRemove")).size(), 0);
+}
+
 void MailSyncTest::testListFolderHierarchy()
 {
     if (!mCapabilities.contains(ResourceCapabilities::Mail::folderhierarchy)) {
@@ -475,8 +499,8 @@ void MailSyncTest::testFailingSync()
     });
 
     VERIFYEXEC(Store::synchronize(query));
-    // Ensure sync fails if resource is misconfigured
-    QTRY_VERIFY(errorReceived);
+    // Ensure sync fails if resource is misconfigured. We have to wait longer than the timeout in imapserverproxy
+    QTRY_VERIFY_WITH_TIMEOUT(errorReceived, 10000);
 }
 
 void MailSyncTest::testSyncUidvalidity()
